"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;
var _assertThisInitialized2 = _interopRequireDefault(require("@babel/runtime/helpers/assertThisInitialized"));
var _inheritsLoose2 = _interopRequireDefault(require("@babel/runtime/helpers/inheritsLoose"));
var _react = _interopRequireDefault(require("react"));
var _propTypes = _interopRequireDefault(require("prop-types"));
var _consolidatedEvents = require("consolidated-events");
var _object = _interopRequireDefault(require("object.values"));
var _document = _interopRequireDefault(require("document.contains"));
var DISPLAY = {
  BLOCK: 'block',
  FLEX: 'flex',
  INLINE: 'inline',
  INLINE_BLOCK: 'inline-block',
  CONTENTS: 'contents'
};
var propTypes = {
  children: _propTypes["default"].node.isRequired,
  onOutsideClick: _propTypes["default"].func.isRequired,
  disabled: _propTypes["default"].bool,
  useCapture: _propTypes["default"].bool,
  display: _propTypes["default"].oneOf((0, _object["default"])(DISPLAY))
};
var defaultProps = {
  disabled: false,
  // `useCapture` is set to true by default so that a `stopPropagation` in the
  // children will not prevent all outside click handlers from firing - maja
  useCapture: true,
  display: DISPLAY.BLOCK
};
var OutsideClickHandler = exports["default"] = /*#__PURE__*/function (_React$Component) {
  (0, _inheritsLoose2["default"])(OutsideClickHandler, _React$Component);
  function OutsideClickHandler() {
    var _this;
    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }
    _this = _React$Component.call.apply(_React$Component, [this].concat(args)) || this;
    _this.onMouseDown = _this.onMouseDown.bind((0, _assertThisInitialized2["default"])(_this));
    _this.onMouseUp = _this.onMouseUp.bind((0, _assertThisInitialized2["default"])(_this));
    _this.setChildNodeRef = _this.setChildNodeRef.bind((0, _assertThisInitialized2["default"])(_this));
    return _this;
  }
  var _proto = OutsideClickHandler.prototype;
  _proto.componentDidMount = function componentDidMount() {
    var _this$props = this.props,
      disabled = _this$props.disabled,
      useCapture = _this$props.useCapture;
    if (!disabled) this.addMouseDownEventListener(useCapture);
  };
  _proto.componentDidUpdate = function componentDidUpdate(_ref) {
    var prevDisabled = _ref.disabled;
    var _this$props2 = this.props,
      disabled = _this$props2.disabled,
      useCapture = _this$props2.useCapture;
    if (prevDisabled !== disabled) {
      if (disabled) {
        this.removeEventListeners();
      } else {
        this.addMouseDownEventListener(useCapture);
      }
    }
  };
  _proto.componentWillUnmount = function componentWillUnmount() {
    this.removeEventListeners();
  }

  // Use mousedown/mouseup to enforce that clicks remain outside the root's
  // descendant tree, even when dragged. This should also get triggered on
  // touch devices.
  ;
  _proto.onMouseDown = function onMouseDown(e) {
    var useCapture = this.props.useCapture;
    var isDescendantOfRoot = this.childNode && (0, _document["default"])(this.childNode, e.target);
    if (!isDescendantOfRoot) {
      if (this.removeMouseUp) {
        this.removeMouseUp();
        this.removeMouseUp = null;
      }
      this.removeMouseUp = (0, _consolidatedEvents.addEventListener)(document, 'mouseup', this.onMouseUp, {
        capture: useCapture
      });
    }
  }

  // Use mousedown/mouseup to enforce that clicks remain outside the root's
  // descendant tree, even when dragged. This should also get triggered on
  // touch devices.
  ;
  _proto.onMouseUp = function onMouseUp(e) {
    var onOutsideClick = this.props.onOutsideClick;
    var isDescendantOfRoot = this.childNode && (0, _document["default"])(this.childNode, e.target);
    if (this.removeMouseUp) {
      this.removeMouseUp();
      this.removeMouseUp = null;
    }
    if (!isDescendantOfRoot) {
      onOutsideClick(e);
    }
  };
  _proto.setChildNodeRef = function setChildNodeRef(ref) {
    this.childNode = ref;
  };
  _proto.addMouseDownEventListener = function addMouseDownEventListener(useCapture) {
    this.removeMouseDown = (0, _consolidatedEvents.addEventListener)(document, 'mousedown', this.onMouseDown, {
      capture: useCapture
    });
  };
  _proto.removeEventListeners = function removeEventListeners() {
    if (this.removeMouseDown) this.removeMouseDown();
    if (this.removeMouseUp) this.removeMouseUp();
  };
  _proto.render = function render() {
    var _this$props3 = this.props,
      children = _this$props3.children,
      display = _this$props3.display;
    return /*#__PURE__*/_react["default"].createElement("div", {
      ref: this.setChildNodeRef,
      style: display !== DISPLAY.BLOCK && (0, _object["default"])(DISPLAY).includes(display) ? {
        display: display
      } : undefined
    }, children);
  };
  return OutsideClickHandler;
}(_react["default"].Component);
OutsideClickHandler.propTypes = propTypes;
OutsideClickHandler.defaultProps = defaultProps;