"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
Object.defineProperty(exports, "DIRECTIONS", {
  enumerable: true,
  get: function get() {
    return _constants.DIRECTIONS;
  }
});
exports["default"] = withDirection;
exports.withDirectionPropTypes = void 0;
var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));
var _inheritsLoose2 = _interopRequireDefault(require("@babel/runtime/helpers/inheritsLoose"));
var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));
var _react = _interopRequireDefault(require("react"));
var _hoistNonReactStatics = _interopRequireDefault(require("hoist-non-react-statics"));
var _deepmerge = _interopRequireDefault(require("deepmerge"));
var _constants = require("./constants");
var _getComponentName = _interopRequireDefault(require("../reactWithStyles/utils/getComponentName"));
var _brcast = _interopRequireDefault(require("./proptypes/brcast"));
var _direction = _interopRequireDefault(require("./proptypes/direction"));
/* eslint-disable react/forbid-foreign-prop-types */
// This higher-order component consumes a string from React context that is
// provided by the DirectionProvider component.
// We can use this to conditionally switch layout/direction for right-to-left layouts.

var contextTypes = (0, _defineProperty2["default"])({}, _constants.CHANNEL, _brcast["default"]);
// set a default direction so that a component wrapped with this HOC can be
// used even without a DirectionProvider ancestor in its react tree.
var defaultDirection = _constants.DIRECTIONS.LTR;

// export for convenience, in order for components to spread these onto their propTypes
var withDirectionPropTypes = exports.withDirectionPropTypes = {
  direction: _direction["default"].isRequired
};
function withDirection(WrappedComponent) {
  var WithDirection = /*#__PURE__*/function (_React$Component) {
    (0, _inheritsLoose2["default"])(WithDirection, _React$Component);
    function WithDirection(props, context) {
      var _this;
      _this = _React$Component.call(this, props, context) || this;
      _this.state = {
        direction: context[_constants.CHANNEL] ? context[_constants.CHANNEL].getState() : defaultDirection
      };
      return _this;
    }
    var _proto = WithDirection.prototype;
    _proto.componentDidMount = function componentDidMount() {
      var _this2 = this;
      if (this.context[_constants.CHANNEL]) {
        // subscribe to future direction changes
        this.channelUnsubscribe = this.context[_constants.CHANNEL].subscribe(function (direction) {
          _this2.setState({
            direction: direction
          });
        });
      }
    };
    _proto.componentWillUnmount = function componentWillUnmount() {
      if (this.channelUnsubscribe) {
        this.channelUnsubscribe();
      }
    };
    _proto.render = function render() {
      var direction = this.state.direction;
      return /*#__PURE__*/_react["default"].createElement(WrappedComponent, (0, _extends2["default"])({}, this.props, {
        direction: direction
      }));
    };
    return WithDirection;
  }(_react["default"].Component);
  var wrappedComponentName = (0, _getComponentName["default"])(WrappedComponent) || 'Component';
  WithDirection.WrappedComponent = WrappedComponent;
  WithDirection.contextTypes = contextTypes;
  WithDirection.displayName = "withDirection(".concat(wrappedComponentName, ")");
  if (WrappedComponent.propTypes) {
    WithDirection.propTypes = (0, _deepmerge["default"])({}, WrappedComponent.propTypes);
    delete WithDirection.propTypes.direction;
  }
  if (WrappedComponent.defaultProps) {
    WithDirection.defaultProps = (0, _deepmerge["default"])({}, WrappedComponent.defaultProps);
  }
  return (0, _hoistNonReactStatics["default"])(WithDirection, WrappedComponent);
}